const svg = d3.select('svg');

const width = +svg.attr('width');
const height = +svg.attr('height');

const render = data => {
    //const clusters = d3.range(0, 3).map((n) => n.toString());
    const clusters = 3;
    const maxIter = 10;
    var iter = 1,
        centroids = [],
        points = [];

    const margin = { top: 20 , right: 10 , bottom: 50, left: 120 };
    const innerWidth = width - margin.left - margin.right;
    const innerHeight = height - margin.top - margin.bottom;

    var colors = d3.scaleOrdinal(d3.schemeCategory10);
    //const colors = d3.scale.category20().range();


    const g = svg.append("g")
        .attr("transform",`translate($)`);

    svg.append("g")
        .append("text")
        .attr("class", "label")
        .attr("transform", "translate(" + (innerWidth) +
            "," + (height + margin.top + margin.bottom) + ")")
        .text("");

    const getEuclidianDistance = (a, b) => {
        var dx = b.x - a.x,
            dy = b.y - a.y;
        return Math.sqrt(Math.pow(dx, 2) + Math.pow(dy, 2));
    }
    const getRandomPoint = (type, fill) => {
        return {
            x: Math.round(Math.random() * width),
            y: Math.round(Math.random() * height),
            type: type,
            fill: fill
        };
    }
    const initializePoints = (num, type) => {
        var result = [];
        for (var i = 0; i < num; i++) {
            var color = colors[i];
            if (type != "centroid") {
                color = "blue";
                //console.log(color);
            }
            var point = getRandomPoint(type, color);
            point.id = point.type + "-" + i;
            result.push(point);
        }
        //console.log(result);
        return result;
    }

    const findClosestCentroid = point => {
        var closest = { i: -1, distance: width * 2 };
        centroids.forEach((d, i) => {
            var distance = getEuclidianDistance(d, point);
            // Only update when the centroid is closer
            if (distance < closest.distance) {
                closest.i = i;
                closest.distance = distance;
            }
        });
        return (centroids[closest.i]);
    }

    const colorizePoints = () => {
        points.forEach(d => {
            var closest = findClosestCentroid(d);
            d.fill = closest.fill;
        });
    }

    const computeClusterCenter = cluster => {
        return [
            d3.mean(cluster, function(d) { return d.x; }),
            d3.mean(cluster, function(d) { return d.y; })
        ];
    }
    const moveCentroids = () => {
        centroids.forEach(d => {
            var cluster = points.filter(function(e) {
                return e.fill === d.fill;
            });
            var center = computeClusterCenter(cluster);
            d.x = center[0];
            d.y = center[1];
        });
    }
    const update = () => {
        const data = points.concat(centroids);
        console.log(data);
        const circle = g.selectAll("circle")
            .data(data);

        circle.enter().append("circle")
            .attr("id", function(d) { return d.id; })
            .attr("class", function(d) { return d.type; })
            .attr("r", 5);

        circle.transition().delay(100).duration(1000)
            .attr("cx", function(d) { return d.x; })
            .attr("cy", function(d) { return d.y; })
            .style("fill", function(d) { return d.fill; });

        circle.exit().remove();
    }

    const setText = text => {
        svg.selectAll(".label").text(text);
    }

    const iterate = () => {
        setText("Iteration " + iter + " of " + maxIter);
        colorizePoints();
        moveCentroids();
        update();
    }

    const initialize = () => {
        centroids = initializePoints(clusters, "centroid");
        //points = initializePoints(data, "point");
        points = data;
        //console.log(points);
        update();

        var interval = setInterval(function() {
            if (iter < maxIter + 1) {
                iterate();
                iter++;
            } else {
                clearInterval(interval);
                setText("Done");
            }
        }, 2 * 100);
    }
    initialize();

};

d3.csv('data.csv').then(data => {
    data.forEach(d => {
        d.x = +d.x;
        d.y = +d.y;
    });
    render(data);
});